from django.urls import path
from . import views


urlpatterns = [
    path('acquireLock', views.acquireLock, name="django_db_lock.acquireLock"),
    path('releaseLock', views.releaseLock, name="django_db_lock.releaseLock"),
    path('getLockInfo', views.getLockInfo, name="django_db_lock.getLockInfo"),
    path('clearExpiredLocks', views.clearExpiredLocks, name="django_db_lock.clearExpiredLocks"),
]



from . import services



class DjangoDbLock(object):
    def __init__(self, lock_name, worker_name, timeout):
        self.lock_name = lock_name
        self.worker_name = worker_name
        self.timeout = timeout
    
    def __enter__(self):
        self.locked = services.acquire_lock(self.lock_name, self.worker_name, self.timeout)
        return self.locked
    
    def __exit__(self, type, value, traceback):
        if self.locked:
            services.release_lock(self.lock_name, self.worker_name)



from django_apiview.views import apiview
from .services import acquire_lock as do_acquire_lock
from .services import release_lock as do_release_lock
from .services import get_lock_info as do_get_lock_info
from .services import clear_expired_locks as do_clear_expired_locks


@apiview
def acquireLock(lockName, workerName, timeout:int):
    result = do_acquire_lock(lockName, workerName, timeout)
    return result


@apiview
def releaseLock(lockName, workerName):
    result = do_release_lock(lockName, workerName)
    return result


@apiview
def getLockInfo(lockName):
    info = do_get_lock_info(lockName)
    return info


@apiview
def clearExpiredLocks():
    do_clear_expired_locks()
    return True



from django.conf import settings

DJANGO_DB_LOCK_AUTO_REGISTER_ADMIN = getattr(settings, "DJANGO_DB_LOCK_AUTO_REGISTER_ADMIN", True)
DJANGO_DB_LOCK_APP_LABEL = getattr(settings, "DJANGO_DB_LOCK_APP_LABEL", "django_db_lock")



import datetime
from django.utils.timezone import now
from django.db.utils import IntegrityError
from django.db import transaction
from .models import Lock


def clear_expired_locks():
    now_time = now()
    with transaction.atomic():
        Lock.objects.filter(expire_time__lte=now_time).delete()


def acquire_lock(lock_name, worker_name, timeout):
    clear_expired_locks()
    lock = Lock()
    lock.lock_name = lock_name
    lock.worker_name = worker_name
    lock.lock_time = now()
    lock.expire_time = lock.lock_time + datetime.timedelta(seconds=timeout)
    try:
        with transaction.atomic():
            lock.save()
        return True
    except IntegrityError:
        return False


def release_lock(lock_name, worker_name):
    clear_expired_locks()
    try:
        lock = Lock.objects.get(lock_name=lock_name, worker_name=worker_name)
        with transaction.atomic():
            lock.delete()
        return True
    except Lock.DoesNotExist:
        return True
    return False


def get_lock_info(lock_name):
    try:
        lock = Lock.objects.get(lock_name=lock_name)
        return {
            "pk": lock.pk,
            "lockName": lock.lock_name,
            "workerName": lock.worker_name,
            "lockTime": lock.lock_time,
            "expireTime": lock.expire_time,
        }
    except Lock.DoesNotExist:
        return None
